/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.io.*;

/**
 * Filter for files required by book. Recognised extensions: tex, ltx, sty, pdf, png, jpeg, jpg,
 * gif.
 */
public class BookContentsFilter implements java.io.FileFilter
{
   public BookContentsFilter()
   {
   }

   /**
    * Determines whether given file is accepted by this filter.
    */
   public boolean accept(File f)
   {
      String name = f.getName();

      if (f.isDirectory())
      {
         return !(name.startsWith(".") || name.startsWith("_")
            || name.startsWith("$") || name.equals("html-src")
            || (new File(f, "index.html")).exists());
      }

      name = name.toLowerCase();

      if (name.endsWith(".tex") ||
          name.endsWith(".ltx") ||
          name.endsWith(".bib") ||
          name.endsWith(".bst") ||
          name.endsWith(".sty") ||
          name.endsWith(".png") ||
          name.endsWith(".jpg") ||
          name.endsWith(".jpeg") ||
          name.endsWith(".gif"))
      {
         return true;
      }

      if (name.endsWith(".pdf"))
      {
         // Is there a corresponding .tex file?

         String basename = name.substring(0, name.lastIndexOf("."));

         File file = new File(f.getParentFile(), basename+".tex");

         if (file.exists())
         {
            return false;
         }

         if (basename.endsWith("-print"))
         {
            basename = basename.substring(0, basename.lastIndexOf("-"));

            file = new File(f.getParentFile(), basename+".tex");

            if (file.exists())
            {
               return false;
            }
         }

         return true;
      }
      
      return false;
   }

}
