/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.util.*;
import java.io.*;
import java.net.*;
import javax.swing.tree.*;

public class BookFile extends FileElement
  implements Comparable<BookFile>
{
   public BookFile(MakeJmlrBookGUI application, File file)
   {
      super(application);
      setName(file.getName());
      parent = null;

      if (file.isDirectory())
      {
         children = new Vector<BookFile>();
      }
      else
      {
         children = null;
      }
   }

   public BookFile(MakeJmlrBookGUI application,
      FileElement parent, String name, boolean isDir)
   {
      super(application);
      this.parent = parent;
      setName(name);
      
      children = isDir ? new Vector<BookFile>() : null;
   }

   public BookDocument toBookDocument()
   {
      BookDocument doc = new BookDocument(app, parent, getName());

      doc.selected = selected;

      if (parent != null)
      {
         for (int i = 0, n = parent.getChildCount(); i < n; i++)
         {
            if (parent.children.get(i) == this)
            {
               parent.children.set(i, doc);
               break;
            }
         }
      }

      return doc;
   }

   public Book toBook()
     throws IOException
   {
      Book book = new Book(app, getRoot(), getFile());

      book.selected = selected;

      if (parent != null)
      {
         parent.children.remove(this);
      }

      return book;
   }

   public File getLocation()
   {
      if (parent == null)
      {
         return new File(".");
      }

      return parent.getFile();
   }

   public File getFile()
   {
      return new File(getLocation(), getName());
   }

   public BookRoot getRoot()
   {
      return parent == null ? null : parent.getRoot();
   }

   public TreeNode getParent()
   {
      return parent;
   }

   public TreePath getTreePath()
   {
      if (parent == null)
      {
         return new TreePath(this);
      }
      else
      {
         return parent.getTreePath().pathByAddingChild(this);
      }
   }

   public int compareTo(BookFile object)
   {
      return getName().compareTo(object.getName());
   }

   protected FileElement parent;
}

