/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.io.*;

import com.dickimawbooks.texparserlib.*;

public class PackageInfo
{
    public PackageInfo(String name, String optionList, File location)
    {
       this.name = name;
       this.options = optionList;
       this.location = location;
    }

    public void addOption(String option)
    {
       if (options == null)
       {
          options = option;
          return;
       }

       if (!options.isEmpty())
       {
          options += ",";
       }

       options += option;
    }

    public boolean hasOption(String option)
    {
       String optionList = ","+options+",";

       return optionList.contains(","+option+",");
    }

    public String getOptions()
    {
       return options == null ? "" : options;
    }

    public boolean isInstalled()
    {
       return location != null;
    }

    public void write(PrintWriter out)
      throws IOException
    {
       out.print("\\usepackage");

       if (options != null && !options.isEmpty())
       {
          out.printf("[%s]", options);
       }

       out.printf("{%s}%n", name);
    }

    public void write(PrintWriter out, TeXParser parser)
      throws IOException
    {
       String esc = new String(Character.toChars(parser.getEscChar()));
       String bg = new String(Character.toChars(parser.getBgChar()));
       String eg = new String(Character.toChars(parser.getEgChar()));

       out.print(String.format("%susepackage", esc));

       if (options != null && !options.isEmpty())
       {
          out.printf("[%s]", options);
       }

       out.printf("%s%s%s%n", bg, name, eg);
    }

    private String name;
    private String options;
    private File location;
}

