/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.awt.Cursor;

public class ProcessPaperThread extends Thread
{
   public ProcessPaperThread(MakeJmlrBookGUI application, BookDocument doc)
   {
      this(application, doc, false);
   }

   public ProcessPaperThread(MakeJmlrBookGUI application, BookDocument doc, boolean clearAux)
   {
      super();

      app = application;
      this.doc = doc;
      this.clearAux = clearAux;
   }

   public void run()
   {
      app.setWaitCursor();
      app.disableTools();

      try
      {
         Book book = app.getBook();

         int year = book.getJmlrYear();
         int volume = book.getJmlrVolume();
         String workshop = book.getTeXWorkShop();
         boolean bookfonts = book.hasBookFontsFile();

         String texArg = "";

         if (year != -1)
         {
            texArg="\\expandafter\\def\\csname @jmlryear\\endcsname{"
                  + year+"}";
         }

         if (volume != -1)
         {
            texArg += "\\expandafter\\def\\csname @jmlrvolume\\endcsname{"
                    + volume+"}";
         }

         if (workshop != null)
         {
            texArg += "\\expandafter\\def\\csname @jmlrworkshop\\endcsname{"
                    + workshop+"}";
         }

         if (bookfonts)
         {
            texArg += "\\AtBeginDocument{\\InputIfFileExists{../bookfonts}{}{}\\normalfont }";
         }

         if (clearAux)
         {
            doc.clearAux();
         }

         doc.clearErrorMessages();
         doc.clearPackageList();
         doc.process(texArg);
      }
      catch (Exception e)
      {
         app.error(e);
      }

      app.workerThreadFinished(this);
      app.enableTools();
      app.setInfo(app.getLabel("message.done"));
      app.setDefaultCursor();
   }

   private BookDocument doc;
   private MakeJmlrBookGUI app;
   private boolean clearAux;
}
