/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;

public class PropertiesDialog extends JDialog
   implements ActionListener
{
   public PropertiesDialog(MakeJmlrBookGUI application)
   {
      super(application, application.getLabel("properties.title"), true);

      app = application;
      properties = app.getProperties();

      String defaultDir = properties.getDefaultDirectory();

      if (defaultDir == null)
      {
         defaultDir = ".";
      }

      File file = new File(defaultDir);

      String setting = properties.getDefaultDirectorySetting();

      if (setting == null)
      {
         setting = "home";
      }

      fileChooser = new JFileChooser(getDir());

      tabbedPane = new JTabbedPane();

      getContentPane().add(tabbedPane, "Center");

      // Book tab

      Box bookPanel = Box.createVerticalBox();
      bookPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
      bookPanel.setAlignmentY(Component.CENTER_ALIGNMENT);

      strictBox = new JCheckBox(app.getLabel("properties.strict"),
         properties.isStrict());
      strictBox.setAlignmentX(Component.LEFT_ALIGNMENT);
      strictBox.setMnemonic(app.getMnemonic("properties.strict"));
      bookPanel.add(strictBox);

      bookPanel.add(Box.createVerticalStrut(30));

      // JMLR W&CP no longer accept HTML files, so logo field
      // no longer required.

      JTextArea logoUrlArea = new JTextArea();
      logoUrlArea.setEditable(false);
      logoUrlArea.setText(app.getLabel("properties.logo.url.info"));
      logoUrlArea.setLineWrap(true);
      logoUrlArea.setWrapStyleWord(true);
      logoUrlArea.setOpaque(false);
      logoUrlArea.setAlignmentX(Component.LEFT_ALIGNMENT);
      //bookPanel.add(logoUrlArea);

      Box logoBox = Box.createHorizontalBox();
      logoBox.setAlignmentX(Component.LEFT_ALIGNMENT);
      //bookPanel.add(logoBox);

      JLabel urlLabel = new JLabel(app.getLabel("properties.logo.url"));
      urlLabel.setDisplayedMnemonic(app.getMnemonic("properties.logo.url"));
      logoBox.add(urlLabel);

      logoUrlField = new JTextField();
      logoUrlField.setAlignmentX(Component.LEFT_ALIGNMENT);
      logoBox.add(logoUrlField);

      urlLabel.setLabelFor(logoUrlField);

      bookPanel.add(Box.createVerticalStrut(30));

      tabbedPane.addTab(app.getLabel("properties.book"),
         new JScrollPane(bookPanel));
      tabbedPane.setMnemonicAt(0, app.getMnemonic("properties.book"));

      // Start up tab

      Box startDirBox = Box.createVerticalBox();
      startDirBox.setAlignmentX(Component.LEFT_ALIGNMENT);
      startDirBox.setAlignmentY(Component.CENTER_ALIGNMENT);

      ButtonGroup bg = new ButtonGroup();

      homeButton = new JRadioButton(app.getLabel("properties.home"));
      homeButton.setActionCommand("disablecustom");
      homeButton.addActionListener(this);
      homeButton.setMnemonic(app.getMnemonic("properties.home"));
      homeButton.setAlignmentX(Component.LEFT_ALIGNMENT);
      homeButton.setAlignmentY(Component.CENTER_ALIGNMENT);
      bg.add(homeButton);
      startDirBox.add(homeButton);

      lastButton = new JRadioButton(app.getLabel("properties.last"));
      lastButton.setActionCommand("disablecustom");
      lastButton.setMnemonic(app.getMnemonic("properties.last"));
      lastButton.addActionListener(this);
      lastButton.setAlignmentX(Component.LEFT_ALIGNMENT);
      lastButton.setAlignmentY(Component.CENTER_ALIGNMENT);
      bg.add(lastButton);
      startDirBox.add(lastButton);

      Box box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);
      box.setAlignmentY(Component.TOP_ALIGNMENT);

      customButton = new JRadioButton(app.getLabel("properties.custom"));
      customButton.setMnemonic(app.getMnemonic("properties.custom"));
      customButton.setActionCommand("enablecustom");
      customButton.addActionListener(this);
      customButton.setAlignmentX(Component.LEFT_ALIGNMENT);
      customButton.setAlignmentY(Component.CENTER_ALIGNMENT);
      customButton.setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));
      bg.add(customButton);
      box.add(customButton);

      customField = new FileField(this, fileChooser, JFileChooser.DIRECTORIES_ONLY);
      box.add(customField);

      if (setting.equals("home"))
      {
         homeButton.setSelected(true);
         customField.setEnabled(false);
      }
      else if (setting.equals("last"))
      {
         homeButton.setSelected(true);
         customField.setEnabled(false);
      }
      else if (setting.equals("custom"))
      {
         customButton.setSelected(true);
         customField.setFileName(file.getAbsolutePath());
      }

      Dimension d = customField.getPreferredSize();
      d.width = (int)customField.getMaximumSize().getWidth();
      customField.setMaximumSize(d);

      startDirBox.add(box);

      startDirBox.add(Box.createVerticalGlue());

      tabbedPane.addTab(app.getLabel("properties.startup"), new JScrollPane(startDirBox));
      tabbedPane.setMnemonicAt(1, app.getMnemonic("properties.startup"));

      // Application paths

      Box appPathBox = Box.createVerticalBox();

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel latexLabel = new JLabel(app.getLabel("properties.path.latex"));
      latexLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      latexLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.latex"));
      box.add(latexLabel);

      Dimension dim = latexLabel.getPreferredSize();
      int maxWidth = (int)dim.getWidth();

      latexField = new FileField(this, properties.getLaTeXApp(), fileChooser);
      latexLabel.setLabelFor(latexField.getTextField());
      box.add(latexField);

      appPathBox.add(box);

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel bibtexLabel = new JLabel(app.getLabel("properties.path.bibtex"));
      bibtexLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      bibtexLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.bibtex"));
      box.add(bibtexLabel);

      dim = bibtexLabel.getPreferredSize();
      maxWidth = (int)Math.max(maxWidth, dim.getWidth());

      bibtexField = new FileField(this, properties.getBibTeXApp(), fileChooser);
      bibtexLabel.setLabelFor(bibtexField.getTextField());
      box.add(bibtexField);

      appPathBox.add(box);

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel gsLabel = new JLabel(app.getLabel("properties.path.gs"));
      gsLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      gsLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.gs"));
      box.add(gsLabel);

      dim = gsLabel.getPreferredSize();
      maxWidth = (int)Math.max(maxWidth, dim.getWidth());

      gsField = new FileField(this, properties.getGSApp(), fileChooser);
      gsLabel.setLabelFor(gsField.getTextField());
      box.add(gsField);

      appPathBox.add(box);

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel kpsewhichLabel = new JLabel(app.getLabel("properties.path.kpsewhich"));
      kpsewhichLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      kpsewhichLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.kpsewhich"));
      box.add(kpsewhichLabel);

      dim = kpsewhichLabel.getPreferredSize();
      maxWidth = (int)Math.max(maxWidth, dim.getWidth());

      kpsewhichField = new FileField(this, properties.getKpsewhichApp(), fileChooser);
      kpsewhichLabel.setLabelFor(kpsewhichField.getTextField());
      box.add(kpsewhichField);

      appPathBox.add(box);

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel epstopdfLabel = new JLabel(app.getLabel("properties.path.epstopdf"));
      epstopdfLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      epstopdfLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.epstopdf"));
      box.add(epstopdfLabel);

      dim = epstopdfLabel.getPreferredSize();
      maxWidth = (int)Math.max(maxWidth, dim.getWidth());

      epstopdfField = new FileField(this, properties.getEpstopdfApp(), fileChooser);
      epstopdfLabel.setLabelFor(epstopdfField.getTextField());
      box.add(epstopdfField);

      //appPathBox.add(box); // epstopdf function not current implemented

      tabbedPane.addTab(app.getLabel("properties.app.paths"), new JScrollPane(appPathBox));
      tabbedPane.setMnemonicAt(tabbedPane.getTabCount()-1,
         app.getMnemonic("properties.app.paths"));

      // Viewers

      Box viewPathBox = Box.createVerticalBox();

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel pdfviewerLabel = new JLabel(app.getLabel("properties.path.pdf"));
      pdfviewerLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      pdfviewerLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.pdf"));
      box.add(pdfviewerLabel);

      dim = pdfviewerLabel.getPreferredSize();
      maxWidth = (int)Math.max(maxWidth, dim.getWidth());

      pdfviewerField = new FileField(this, properties.getPdfViewer(), fileChooser);
      pdfviewerLabel.setLabelFor(pdfviewerField.getTextField());
      box.add(pdfviewerField);

      viewPathBox.add(box);

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel textviewerLabel = new JLabel(app.getLabel("properties.path.text"));
      textviewerLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      textviewerLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.text"));
      box.add(textviewerLabel);

      dim = textviewerLabel.getPreferredSize();
      maxWidth = (int)Math.max(maxWidth, dim.getWidth());

      textviewerField = new FileField(this, properties.getTextViewer(), fileChooser);
      textviewerLabel.setLabelFor(textviewerField.getTextField());
      box.add(textviewerField);

      viewPathBox.add(box);

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel htmlviewerLabel = new JLabel(app.getLabel("properties.path.html"));
      htmlviewerLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      htmlviewerLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.html"));
      box.add(htmlviewerLabel);

      dim = htmlviewerLabel.getPreferredSize();
      maxWidth = (int)Math.max(maxWidth, dim.getWidth());

      htmlviewerField = new FileField(this, properties.getHtmlViewer(), fileChooser);
      htmlviewerLabel.setLabelFor(htmlviewerField.getTextField());
      box.add(htmlviewerField);

      viewPathBox.add(box);

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel imageviewerLabel = new JLabel(app.getLabel("properties.path.image"));
      imageviewerLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
      imageviewerLabel.setDisplayedMnemonic(app.getMnemonic("properties.path.image"));
      box.add(imageviewerLabel);

      dim = imageviewerLabel.getPreferredSize();
      maxWidth = (int)Math.max(maxWidth, dim.getWidth());

      imageviewerField = new FileField(this, properties.getImageViewer(), fileChooser);
      imageviewerLabel.setLabelFor(imageviewerField.getTextField());
      box.add(imageviewerField);

      viewPathBox.add(box);

      tabbedPane.addTab(app.getLabel("properties.view.paths"), new JScrollPane(viewPathBox));
      tabbedPane.setMnemonicAt(tabbedPane.getTabCount()-1,
         app.getMnemonic("properties.view.paths"));

      // Process Settings Tab

      Box processBox = Box.createVerticalBox();
      processBox.setAlignmentX(Component.LEFT_ALIGNMENT);

      processBox.add(new Box.Filler(new Dimension(0,0),
        new Dimension(0,10),
        new Dimension(0,Integer.MAX_VALUE)));

      int maxProcWidth = 0;

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);

      JLabel maxProcessTimeLabel = new JLabel(MakeJmlrBookGUI.getLabel(
         "properties.max_process_time"));
      maxProcessTimeLabel.setDisplayedMnemonic(MakeJmlrBookGUI.getMnemonic(
         "properties.max_process_time"));
      box.add(maxProcessTimeLabel);

      maxProcessTimeModel = new SpinnerNumberModel();
      maxProcessTimeModel.setMinimum(new Integer(0));
      JSpinner maxProcessTimeSpinner = new JSpinner(maxProcessTimeModel);
      maxProcessTimeLabel.setLabelFor(maxProcessTimeSpinner);

      box.add(maxProcessTimeSpinner);

      box.add(Box.createHorizontalStrut(5));
      box.add(new JLabel(app.getLabel("properties.minutes")));

      maxProcWidth = (int)Math.max(maxProcWidth, 
         maxProcessTimeLabel.getPreferredSize().getWidth());

      box.add(Box.createHorizontalGlue());

      processBox.add(box);

      processBox.add(Box.createVerticalStrut(12));

      loggingBox = new JCheckBox(app.getLabel("properties.logging"),
         properties.isLogging());
      loggingBox.setAlignmentX(Component.LEFT_ALIGNMENT);
      loggingBox.setMnemonic(app.getMnemonic("properties.logging"));
      processBox.add(loggingBox);
      processBox.add(new JLabel(app.getLabelWithValue("properties.logging.file",
         ""+properties.getLoggingFile())));
      processBox.add(new JLabel(app.getLabel("properties.logging.info")));

      processBox.add(Box.createVerticalStrut(12));

      bg = new ButtonGroup();

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);
      processBox.add(box);
      box.add(new JLabel(app.getLabel("properties.details")));

      detailsIconButton = new JRadioButton(app.getLabel("properties.details.icon"));
      detailsIconButton.setMnemonic(app.getMnemonic("properties.details.icon"));
      bg.add(detailsIconButton);
      box.add(detailsIconButton);

      detailsTextButton = new JRadioButton(app.getLabel("properties.details.text"));
      detailsTextButton.setMnemonic(app.getMnemonic("properties.details.text"));
      bg.add(detailsTextButton);
      box.add(detailsTextButton);

      detailsIconTextButton = new JRadioButton(app.getLabel("properties.details.icontext"));
      detailsIconTextButton.setMnemonic(app.getMnemonic("properties.details.icontext"));
      bg.add(detailsIconTextButton);
      box.add(detailsIconTextButton);

      processBox.add(new Box.Filler(new Dimension(0,0),
        new Dimension(0,10),
        new Dimension(0,Integer.MAX_VALUE)));

      tabbedPane.addTab(app.getLabel("properties.process_settings"), 
         new JScrollPane(processBox));
      tabbedPane.setMnemonicAt(tabbedPane.getTabCount()-1,
         app.getMnemonic("properties.process_settings"));

      dim = latexLabel.getPreferredSize();
      dim.width = maxWidth;
      latexLabel.setPreferredSize(dim);

      dim = bibtexLabel.getPreferredSize();
      dim.width = maxWidth;
      bibtexLabel.setPreferredSize(dim);

      dim = gsLabel.getPreferredSize();
      dim.width = maxWidth;
      gsLabel.setPreferredSize(dim);

      dim = kpsewhichLabel.getPreferredSize();
      dim.width = maxWidth;
      kpsewhichLabel.setPreferredSize(dim);

      dim = epstopdfLabel.getPreferredSize();
      dim.width = maxWidth;
      epstopdfLabel.setPreferredSize(dim);

      dim = pdfviewerLabel.getPreferredSize();
      dim.width = maxWidth;
      pdfviewerLabel.setPreferredSize(dim);

      dim = textviewerLabel.getPreferredSize();
      dim.width = maxWidth;
      textviewerLabel.setPreferredSize(dim);

      dim = htmlviewerLabel.getPreferredSize();
      dim.width = maxWidth;
      htmlviewerLabel.setPreferredSize(dim);

      dim = imageviewerLabel.getPreferredSize();
      dim.width = maxWidth;
      imageviewerLabel.setPreferredSize(dim);

      JPanel buttonPanel = new JPanel();
      getContentPane().add(buttonPanel, "South");

      buttonPanel.add(JMLRResources.createOkayButton(this));
      buttonPanel.add(JMLRResources.createCancelButton(this));

      JButton helpButton = new JButton(app.getLabel("button.help"));
      helpButton.setMnemonic(app.getMnemonic("button.help"));
      app.enableHelpOnButton(helpButton, "sec:settings");
      buttonPanel.add(helpButton);

      pack();

      dim = getSize();
      dim.width += 30;
      dim.height += 30;
      setSize(dim);

      setLocationRelativeTo(app);
   }

   public void display()
   {
      latexField.setFileName(properties.getLaTeXApp());
      bibtexField.setFileName(properties.getBibTeXApp());
      gsField.setFileName(properties.getGSApp());
      kpsewhichField.setFileName(properties.getKpsewhichApp());
      epstopdfField.setFileName(properties.getEpstopdfApp());
      pdfviewerField.setFileName(properties.getPdfViewer());
      textviewerField.setFileName(properties.getTextViewer());
      htmlviewerField.setFileName(properties.getHtmlViewer());
      strictBox.setSelected(properties.isStrict());
      loggingBox.setSelected(properties.isLogging());

      boolean showText = properties.showDetailsText();
      boolean showIcon = properties.showDetailsIcon();

      if (showText && showIcon)
      {
         detailsIconTextButton.setSelected(true);
      }
      else if (showIcon)
      {
         detailsIconButton.setSelected(true);
      }
      else
      {
         detailsTextButton.setSelected(true);
      }

      maxProcessTimeModel.setValue(new Long(properties.getMaxProcessTime()/60000L));
      String logoUrl = properties.getLogoUrl();
      logoUrlField.setText(logoUrl == null ? "" : logoUrl);

      setVisible(true);
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("disablecustom"))
      {
         customField.setEnabled(false);
      }
      else if (action.equals("enablecustom"))
      {
         customField.setEnabled(true);
         customField.requestFocusInWindow();
      }
      else if (action.equals("cancel"))
      {
         setVisible(false);
      }
      else if (action.equals("okay"))
      {
         properties.setLaTeXApp(latexField.getFileName());
         properties.setBibTeXApp(bibtexField.getFileName());
         properties.setGSApp(gsField.getFileName());
         properties.setKpsewhichApp(kpsewhichField.getFileName());
         properties.setEpstopdfApp(epstopdfField.getFileName());
         properties.setPdfViewer(pdfviewerField.getFileName());
         properties.setTextViewer(textviewerField.getFileName());
         properties.setHtmlViewer(htmlviewerField.getFileName());
         properties.setImageViewer(imageviewerField.getFileName());
         properties.setStrict(strictBox.isSelected());
         properties.setLogoUrl(logoUrlField.getText());
         properties.setMaxProcessTime(maxProcessTimeModel.getNumber().longValue()*60000L);

         if (homeButton.isSelected())
         {
            properties.setDefaultHomeDir();
         }
         else if (lastButton.isSelected())
         {
            properties.setDefaultLastDir();
         }
         else
         {
            String fileName  = customField.getFileName();

            if (fileName.equals(""))
            {
               JMLRResources.error(this, app.getLabel("error.prop.missing.custom"));
               return;
            }

            File file = new File(fileName);

            if (!file.isDirectory())
            {
               JMLRResources.error(this,
                  app.getLabelWithValue("error.dir.not", fileName));
            }

            properties.setDefaultCustomDir(fileName);
         }
         
         properties.enableLogging(loggingBox.isSelected());

         properties.setShowDetailsIcon(detailsIconButton.isSelected()
           || detailsIconTextButton.isSelected());

         properties.setShowDetailsText(detailsTextButton.isSelected()
           || detailsIconTextButton.isSelected());

         setVisible(false);
      }
   }

   private File getDir()
   {
      String latexApp = properties.getLaTeXApp();
      String bibtexApp = properties.getBibTeXApp();

      if ((latexApp == null || latexApp.equals(""))
        &&(bibtexApp == null || bibtexApp.equals("")))
      {
         try
         {
            return (new File(app.kpsewhich("texmf.cnf", false))).getParentFile();
         }
         catch (IOException e)
         {
         }
         catch (InterruptedException e)
         {
         }
      }

      return new File(properties.getDefaultDirectory());
   }

   private MakeJmlrBookGUI app;
   private JMLRBookProperties properties;

   private JTabbedPane tabbedPane;

   private JFileChooser fileChooser;

   private JRadioButton homeButton, lastButton, customButton;

   private JCheckBox strictBox;

   private JCheckBox loggingBox;

   private JRadioButton detailsIconButton, detailsTextButton,
     detailsIconTextButton;

   private FileField customField, latexField, bibtexField,
       gsField, kpsewhichField, epstopdfField,
       pdfviewerField, textviewerField, htmlviewerField,
       imageviewerField;

   private JTextField logoUrlField;

   private SpinnerNumberModel maxProcessTimeModel;

}
