/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.nio.charset.Charset;
import java.io.File;
import java.util.Vector;

import com.dickimawbooks.texparserlib.Group;
import com.dickimawbooks.texparserlib.latex.LaTeXFile;

public class ImportFileAttributes
{
   public ImportFileAttributes(ProcessPanel panel, File subDir, String tag)
   {
      this(panel, subDir, tag, null, null, null);
   }

   public ImportFileAttributes(ProcessPanel panel, 
     String tag, Charset charset,
     Vector<LaTeXFile> styList, Vector<Group> authors)
   {
      this(panel, null, tag, charset, styList, authors);
   }

   public ImportFileAttributes(ProcessPanel panel, File subDir,
     String tag, Charset charset,
     Vector<LaTeXFile> styList, Vector<Group> authors)
   {
      this.panel = panel;
      this.subDir = subDir;
      this.tag = tag;
      this.charset = charset;
      this.styList = styList;
      this.authors = authors;
   }

   public void setTag(String tag)
   {
      this.tag = tag;
   }

   public Charset getCharset()
   {
      return charset;
   }

   public String getTag()
   {
      return tag;
   }

   public Vector<LaTeXFile> getPackages()
   {
      return styList;
   }

   public Vector<Group> getAuthors()
   {
      return authors;
   }

   public File getToImportFile()
   {
      File parent = subDir;

      if (parent == null)
      {
         parent = new File(panel.getBookDir(), tag);
      }

      return new File(parent, tag+".tex");
   }

   public String toString()
   {
      return String.format("%s[tag=%s,subdir=%s,charset=%s,author count=%d,sty count:%d]",
        getClass().getSimpleName(),
        tag, subDir, charset,
        authors == null ? 0 : authors.size(),
        styList==null?0:styList.size());
   }

   private String tag;
   private Charset charset;
   private Vector<LaTeXFile> styList;
   private Vector<Group> authors;
   private ProcessPanel panel;
   private File subDir;
}
