/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.util.Vector;
import java.io.File;
import java.io.IOException;

import com.dickimawbooks.texparserlib.*;
import com.dickimawbooks.texparserlib.generic.*;
import com.dickimawbooks.texparserlib.latex.*;
import com.dickimawbooks.texparserlib.latex2latex.*;

import com.dickimawbooks.makejmlrbookgui.*;

public class NewBookParser extends LaTeX2LaTeX
{
   public NewBookParser(TeXApp texApp, File outDir)
     throws IOException
   {
      super(texApp, outDir, true);
   }

   public Other getOther(int charCode)
   {
      return new NewBookOther(charCode);
   }

   public void setSignOffFound(boolean found)
   {
      doSignOff = !found;
   }

   public void newcommand(byte overwrite,
     String type, String csName, boolean isShort,
     int numParams, TeXObject defValue, TeXObject definition)
    throws IOException
   {
      if (Accent.isAccentCommand(csName))
      {
         String message = MakeJmlrBookGUI.getLabelWithValue(
            "warning.accent_redef", csName);

         writeCodePoint(getParser().getCommentChar());
         write(String.format(" %s%n", message));
         getTeXApp().warning(getParser(), message);
      }

      super.newcommand(overwrite, type, csName, isShort,
        numParams, defValue, definition);
   }

   protected boolean doSignOff=true;
}

class NewBookOther extends L2LOther
{
   public NewBookOther(int charCode)
   {
      super(charCode);
   }

   public Object clone()
   {
      return new NewBookOther(getCharCode());
   }

   public void process(TeXParser parser) throws IOException
   {
      Writeable writeable = parser.getListener().getWriteable();

      for (int i = 0; i < CHAR_MAPS.length; i++)
      {
         if (CHAR_MAPS[i][0].codePointAt(0) == getCharCode())
         {
            parser.getListener().getTeXApp().substituting(
               parser, CHAR_MAPS[i][0], CHAR_MAPS[i][1]);
            writeable.write(CHAR_MAPS[i][1]);
            return;
         }
      }

      if (getCharCode() == (int)'"')
      {
         if (parser.size() == 0)
         {
            parser.fetchNext();
         }

         TeXObject obj = parser.peekStack();

         if (obj instanceof Letter)
         {
            parser.getListener().getTeXApp().substituting(
               parser, "\"", "``");
            writeable.write("``");
         }
         else
         {
            parser.getListener().getTeXApp().substituting(
              parser, "\"", "''");
            writeable.write("''");
         }

         return;
      }

      super.process(parser);
   }

   public void process(TeXParser parser, TeXObjectList stack) throws IOException
   {
      Writeable writeable = parser.getListener().getWriteable();

      for (int i = 0; i < CHAR_MAPS.length; i++)
      {
         if (CHAR_MAPS[i][0].codePointAt(0) == getCharCode())
         {
            parser.getListener().getTeXApp().substituting(
              parser, CHAR_MAPS[i][0], CHAR_MAPS[i][1]);
            writeable.write(CHAR_MAPS[i][1]);
            return;
         }
      }

      if (getCharCode() == (int)'"')
      {
         TeXObject obj = stack.peek();

         if (obj instanceof Letter)
         {
            parser.getListener().getTeXApp().substituting(
               parser, "\"", "``");
            writeable.write("``");
         }
         else
         {
            parser.getListener().getTeXApp().substituting(
               parser, "\"", "''");
            writeable.write("''");
         }

         return;
      }

      super.process(parser, stack);
   }

   private static final String[][] CHAR_MAPS =
    {
       new String[] {"‘", "`"},
       new String[] {"’", "'"},
       new String[] {"“", "``"},
       new String[] {"”", "''"},
       new String[] {"–", "--"},
       new String[] {"—", "---"},
       new String[] {"‐", "-"},
       new String[] {"‑", "-"},
       new String[] {"À", "\\`{A}"},
       new String[] {"Á", "\\'{A}"},
       new String[] {"Â", "\\^{A}"},
       new String[] {"Ã", "\\~{A}"},
       new String[] {"Ä", "\\\"{A}"},
       new String[] {"Å", "\\r{A}"},
       new String[] {"Æ", "\\AE{}"},
       new String[] {"Ç", "\\c{C}"},
       new String[] {"È", "\\`{E}"},
       new String[] {"É", "\\'{E}"},
       new String[] {"Ê", "\\^{E}"},
       new String[] {"Ë", "\\\"{E}"},
       new String[] {"Ì", "\\`{I}"},
       new String[] {"Í", "\\'{I}"},
       new String[] {"Î", "\\^{I}"},
       new String[] {"Ï", "\\\"{I}"},
       new String[] {"Ñ", "\\~{N}"},
       new String[] {"Ò", "\\`{O}"},
       new String[] {"Ó", "\\'{O}"},
       new String[] {"Ô", "\\^{O}"},
       new String[] {"Õ", "\\~{O}"},
       new String[] {"Ö", "\\\"{O}"},
       new String[] {"Ø", "\\O{}"},
       new String[] {"Ù", "\\`{U}"},
       new String[] {"Ú", "\\'{U}"},
       new String[] {"Û", "\\^{U}"},
       new String[] {"Ü", "\\\"{U}"},
       new String[] {"Ý", "\\'{Y}"},
       new String[] {"ß", "\\ss{}"},
       new String[] {"à", "\\`{a}"},
       new String[] {"á", "\\'{a}"},
       new String[] {"â", "\\^{a}"},
       new String[] {"ã", "\\~{a}"},
       new String[] {"ä", "\\\"{a}"},
       new String[] {"å", "\\r{a}"},
       new String[] {"æ", "\\ae{}"},
       new String[] {"ç", "\\c{c}"},
       new String[] {"è", "\\`{e}"},
       new String[] {"é", "\\'{e}"},
       new String[] {"ê", "\\^{e}"},
       new String[] {"ë", "\\\"{e}"},
       new String[] {"ì", "\\`{i}"},
       new String[] {"í", "\\'{i}"},
       new String[] {"î", "\\^{i}"},
       new String[] {"ï", "\\\"{i}"},
       new String[] {"ñ", "\\~{n}"},
       new String[] {"ò", "\\`{o}"},
       new String[] {"ó", "\\'{o}"},
       new String[] {"ô", "\\^{o}"},
       new String[] {"õ", "\\~{o}"},
       new String[] {"ö", "\\\"{o}"},
       new String[] {"ø", "\\o{}"},
       new String[] {"ù", "\\`{u}"},
       new String[] {"ú", "\\'{u}"},
       new String[] {"û", "\\^{u}"},
       new String[] {"ü", "\\\"{u}"},
       new String[] {"ý", "\\'{y}"},
       new String[] {"ÿ", "\\\"{y}"},
       new String[] {"ﬀ", "ff"},
       new String[] {"ﬁ", "fi"},
       new String[] {"ﬂ", "fl"},
       new String[] {"ﬃ", "ffi"},
       new String[] {"ﬄ", "ffl"},
    };

}

