/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Iterator;

import javax.swing.*;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.dickimawbooks.texparserlib.bib.Contributor;

import com.dickimawbooks.makejmlrbookgui.*;

public class TitleContributorsField extends JPanel 
  implements TitleField,ActionListener,ListSelectionListener
{
   public TitleContributorsField(String parentTag, String tag,
      final String rowLabel)
   {
      super(new FlowLayout(FlowLayout.LEADING, 0, 0));
      setAlignmentX(LEFT_ALIGNMENT);
      this.rowLabel = rowLabel;

      data = new Vector<Contributor>();
      data.add(new Contributor());

      model = new AbstractTableModel()
      {
         public int getColumnCount() {return 3;}
         public int getRowCount() {return data.size();}

         public boolean isCellEditable(int row, int col)
         {
            return col > 0;
         }

         public Object getValueAt(int row, int col)
         {
            if (col == 0) return getRowName(row);

            Contributor contrib = data.get(row);

            return col == 1 ? contrib.getForenames() : contrib.getSurname();
         }

         public void setValueAt(Object value, int row, int col)
         {
            if (col == 0) return;

            Contributor contrib = data.get(row);

            if (col == 1)
            {
               contrib.setForenames(value.toString());
            }
            else
            {
               contrib.setSurname(value.toString());
            }
         }

         public String getColumnName(int column)
         {
            switch (column)
            {
               case 0: return "";
               case 1: return MakeJmlrBookGUI.getLabel(rowLabel+".forenames");
               case 2: return MakeJmlrBookGUI.getLabel(rowLabel+".surname");
            }

            return "";
         }
      };

      table = new JTable(model);

      table.setAlignmentX(LEFT_ALIGNMENT);
      table.setColumnSelectionAllowed(false);
      table.setRowSelectionAllowed(true);
      //table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
      table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
      table.setDragEnabled(false);

      ListSelectionModel selectionModel = table.getSelectionModel();
      selectionModel.setSelectionMode(
         ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

      selectionModel.addListSelectionListener(this);

      JScrollPane sp = new JScrollPane(table);
      sp.setBorder(BorderFactory.createEmptyBorder());
      sp.setAlignmentX(LEFT_ALIGNMENT);

      Dimension dim = sp.getPreferredSize();
      dim.height = (int)table.getPreferredSize().getHeight() 
        + 4*table.getRowHeight();
      sp.setPreferredSize(dim);

      add(sp);

      JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
      buttonPanel.setAlignmentX(LEFT_ALIGNMENT);

      JButton addButton = JMLRResources.createAddButton(this);
      buttonPanel.add(addButton);

      removeButton = JMLRResources.createRemoveButton(this);
      removeButton.setEnabled(false);
      buttonPanel.add(removeButton);

      String label = parentTag+"."+tag;

      labelComponent = Box.createVerticalBox();
      labelComponent.setAlignmentX(LEFT_ALIGNMENT);

      JLabel jLabel = new JLabel(MakeJmlrBookGUI.getLabel(label));
      jLabel.setAlignmentX(LEFT_ALIGNMENT);
      jLabel.setDisplayedMnemonic(MakeJmlrBookGUI.getMnemonic(label));

      labelComponent.add(jLabel);
      labelComponent.add(buttonPanel);
   }

   public int getIntValue()
   {
      return 0;
   }

   public String getRowName(int row)
   {
      return MakeJmlrBookGUI.getLabelWithValue(rowLabel, row+1);
   }

   private void addElement(Object object)
   {
      if (object instanceof Contributor)
      {
         Contributor contrib = (Contributor)object;

         String forename = contrib.getForenames();

         if (forename == null) forename = "";

         String von = contrib.getVonPart();
         String surname = contrib.getSurname();
         String suffix = contrib.getSuffix();

         StringBuffer buff = new StringBuffer();

         if (von != null)
         {
            buff.append(von);

            if (surname != null)
            {
               buff.append(" ");
            }
            else if (suffix != null)
            {
               buff.append(", ");
            }
         }

         if (surname != null)
         {
            buff.append(surname);

            if (suffix != null) buff.append(", ");
         }

         if (suffix != null)
         {
            buff.append(suffix);
         }

         surname = buff.toString();

         if (surname.matches("^\\{[^\\}]+\\}$"))
         {
            surname = surname.substring(1, surname.length()-1);
         }

         data.add(new Contributor(forename, surname));
      }
      else
      {
         String name = object.toString().trim();

         int idx = name.lastIndexOf(" ");

         if (idx == -1)
         {
            data.add(new Contributor("", name));
         }
         else
         {
            data.add(new Contributor(name.substring(0, idx),
              name.substring(idx+1)));
         }
      }
   }

   public void setFieldValue(Object object)
   {
      data.clear();

      if (object instanceof java.util.List)
      {
         for (Iterator it = ((java.util.List)object).iterator();
              it.hasNext(); )
         {
            addElement(it.next());
         }
      }
      else
      {
         addElement(object);
      }

      model.fireTableDataChanged();
      table.revalidate();
   }

   public Vector<String> getList()
   {
      Vector<String> list = new Vector<String>(data.size());

      for (int i = 0; i < data.size(); i++)
      {
         list.add(getElement(i));
      }

      return list;
   }

   public Vector<Contributor> getContributors()
   {
      return data;
   }

   private String getElement(int i)
   {
      if (i < 0 || i > data.size()) return "";

      Contributor contrib = data.get(i);

      String forename = contrib.getForenames();
      String surname = contrib.getSurname();

      if (surname.contains(" "))
      {
         surname = "{"+surname+"}";
      }

      return String.format("%s %s", forename, surname);
   }

   public JComponent getComponent()
   {
      return this;
   }

   public JComponent getLabel()
   {
      return labelComponent;
   }

   public String getText()
   {
      return getElement(0);
   }

   public int getNumValues()
   {
      return data.size();
   }

   public boolean isEmpty()
   {
      if (data.isEmpty()) return true;

      for (Contributor contrib : data)
      {
         String forenames = contrib.getForenames();
         String surname = contrib.getSurname();

         if (!(forenames == null || forenames.isEmpty())
          || !(surname == null || surname.isEmpty()))
         {
            return false;
         }
      }

      return true;
   }

   public void updateRemoveButton()
   {
      int rowCount = table.getRowCount();
      int selRowCount = table.getSelectedRowCount();

      removeButton.setEnabled(
        rowCount > 1 && selRowCount > 0 && selRowCount < rowCount);
   }

   public void valueChanged(ListSelectionEvent evt)
   {
      int colIdx = table.getSelectedColumn();

      if (colIdx == 0)
      {
         int rowIdx = table.getSelectedRow();

         table.changeSelection(rowIdx, 1, false, false);
      }
      else
      {
         updateRemoveButton();
      }

   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("add"))
      {
         addRow();
         updateRemoveButton();
      }
      else if (action.equals("remove"))
      {
         int[] rows = table.getSelectedRows();

         if (rows.length == 1)
         {
            data.remove(rows[0]);
         }
         else if (rows.length > 0)
         {
            Vector<Contributor> sel = new Vector<Contributor>(rows.length);

            for (int i = 0; i < rows.length; i++)
            {
               sel.add(data.get(rows[i]));
            }

            data.remove(sel);
         }

         updateRemoveButton();
      }
   }

   public void addRow()
   {
      int idx = data.size();
      data.add(new Contributor("", ""));

      model.fireTableRowsInserted(idx, idx);
      revalidate();
   }

   private JTable table;
   private AbstractTableModel model;

   private Vector<Contributor> data;

   private JButton removeButton;
   private String rowLabel;
   private JComponent labelComponent;
}
