/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import javax.swing.*;

import com.dickimawbooks.texparserlib.bib.Contributor;

import com.dickimawbooks.makejmlrbookgui.*;

public class TitleFileField extends FileField implements TitleField
{
   public TitleFileField(String parentTag, String tag,
      final String filterType, File defaultDir)
   {
      super(null, null);
      setAlignmentX(LEFT_ALIGNMENT);
      setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));

      setFileChooser(createFileChooser(filterType, defaultDir));

      String label = parentTag+"."+tag;

      jLabel = new JLabel(MakeJmlrBookGUI.getLabel(label));
      jLabel.setDisplayedMnemonic(MakeJmlrBookGUI.getMnemonic(label));
      jLabel.setLabelFor(getTextField());

      String tooltip = MakeJmlrBookGUI.getLabelWithAlt(label+".tooltip", null);

      if (tooltip != null)
      {
         getTextField().setToolTipText(tooltip);
         jLabel.setToolTipText(tooltip);
      }

      Dimension dim = getMaximumSize();
      dim.height = (int)getPreferredSize().getHeight();
      setMaximumSize(dim);
   }

   private JFileChooser createFileChooser(String filterType, File defaultDir)
   {
      JFileChooser chooser = new JFileChooser();

      if (filterType.equals("image"))
      {
         ImageFileFilter filter = new ImageFileFilter();
         chooser.addChoosableFileFilter(filter);
         chooser.setFileFilter(filter);
      }
      else if (filterType.equals("tex"))
      {
         TeXFileFilter filter = new TeXFileFilter();
         chooser.addChoosableFileFilter(filter);
         chooser.setFileFilter(filter);
      }

      chooser.setCurrentDirectory(defaultDir);

      return chooser;
   }

   public JComponent getComponent()
   {
      return this;
   }

   public JComponent getLabel()
   {
      return jLabel;
   }

   public int getNumValues()
   {
      return 1;
   }

   public boolean isEmpty()
   {
      return getFileName().isEmpty();
   }

   public String getText()
   {
      return getFileName();
   }

   public void setFieldValue(Object object)
   {
      if (object instanceof File)
      {
         setFile((File)object);
      }
      else
      {
         setFileName(object.toString());
      }
   }

   public Vector<String> getList()
   {
      Vector<String> list = new Vector<String>(1);
      list.add(getText());

      return list;
   }

   public Vector<Contributor> getContributors()
   {
      return null;
   }

   public int getIntValue()
   {
      return 0;
   }

   private JLabel jLabel;
}

